
/**
 * Class that represents a sound.  This class is used by the students
 * to extend the capabilities of SimpleSound. 
 * 
 * @author Barbara Ericson
 */
public class Sound extends SimpleSound
{
  
  /////////////// consructors ////////////////////////////////////
  
  /**
   * Constructor that takes a file name
   * @param fileName the name of the file to read the sound from
   */
  public Sound(String fileName)
  {
    // let the parent class handle setting the file name
    super(fileName);
  }
  
  /**
   * Constructor that takes the number of seconds that this
   * sound will have
   * @param numSeconds the number of seconds desired
   */
  public Sound (int numSeconds)
  {
    // let the parent class handle this
    super(numSeconds);
  }
  
  ////////////////// methods ////////////////////////////////////
  
  /**
   * Method to return the string representation of this sound
   * @return a string with information about this sound
   */
  public String toString()
  {
    String output = "Sound";
    String fileName = getFileName();
    
    // if there is a file name then add that to the output
    if (fileName != null)
      output = output + " file: " + fileName;
    
    // add the length in frames
    output = output + " number of samples: " + getLengthInFrames();
    
    return output;
  }
            
  /**
   * Method to reverse a sound.
   **/
  public Sound reverse()
  {
    Sound target = new Sound(getLength());
    int sampleValue;
    
    for (int srcIndex=0,trgIndex=getLength()-1;
         srcIndex < getLength();
         srcIndex++,trgIndex--)
    {
      sampleValue = this.getSampleValueAt(srcIndex);
      target.setSampleValueAt(trgIndex,sampleValue);
    };
    return target;
  }
  
  /**
   * Return this sound appended with the input sound
   * @param appendSound sound to append to this
   **/
  public Sound append(Sound appendSound) {
    Sound target = new Sound(getLength()+appendSound.getLength());
    int sampleValue;
    
    // Copy this sound in
    for (int srcIndex=0,trgIndex=0;
         srcIndex < getLength();
         srcIndex++,trgIndex++)
    {
      sampleValue = this.getSampleValueAt(srcIndex);
      target.setSampleValueAt(trgIndex,sampleValue);
    };
    
    // Copy appendSound in to target
    for (int srcIndex=0,trgIndex=getLength();
         srcIndex < appendSound.getLength();
         srcIndex++,trgIndex++)
    {
      sampleValue = appendSound.getSampleValueAt(srcIndex);
      target.setSampleValueAt(trgIndex,sampleValue);
    };

    return target;
  }
  
  /**
   * Mix the input sound with this sound, with percent ratio of input.
   * Use mixIn sound up to length of this sound.
   * Return mixed sound.
   * @param mixIn sound to mix in
   * @param ratio how much of input mixIn to mix in
   **/
  public Sound mix(Sound mixIn, double ratio){
        Sound target = new Sound(getLength());
        
        int sampleValue, mixValue,newValue;
    
        // Copy this sound in
        for (int srcIndex=0,trgIndex=0;
             srcIndex < getLength() && srcIndex < mixIn.getLength();
             srcIndex++,trgIndex++)
        {
          sampleValue = this.getSampleValueAt(srcIndex);
          mixValue = mixIn.getSampleValueAt(srcIndex);
          newValue = (int)(ratio*mixValue) + (int)((1.0-ratio)*sampleValue);
          target.setSampleValueAt(trgIndex,newValue);
        };
        return target;
  }
    
  /**
   * Scale up or down a sound by the given factor
   * (1.0 returns the same, 2.0 doubles the length, and 0.5 halves the length)
   * @param factor ratio to increase or decrease
   **/
  public Sound scale(double factor){
        Sound target = new Sound((int) (factor * (1+getLength())));
        int sampleValue;
    
        // Copy this sound in
        for (double srcIndex=0.0,trgIndex=0;
             srcIndex < getLength();
             srcIndex+=(1/factor),trgIndex++)
        {
          sampleValue = this.getSampleValueAt((int)srcIndex);
          target.setSampleValueAt((int) trgIndex,sampleValue);
        };
        return target;
  }
  
} // end of class Sound put all new methods before this last curly brace