    
/*************************************************************************
 *  Creates a Turtle on an input 
 *
 *************************************************************************/

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import javax.swing.*;
import java.awt.image.*;


public class Turtle-lowerleft  {

    private Picture myPicture;              // the picture that we're drawing on
    private Graphics2D myGraphics;
    JFrame myWindow;

    private double x = 0.0, y = 0.0;           // turtle is at coordinate (x, y)
    private int height, width;
    private double heading = 0.0;          // facing this many degrees counterclockwise
    private Color foreground = Color.black;    // foreground color
    private boolean penDown = true;

    // turtles are created on pictures
    public Turtle(Picture newPicture) {
      myPicture = newPicture;
      myGraphics = (Graphics2D) myPicture.getBufferedImage().createGraphics();
      myGraphics.setColor(foreground);
      //BufferedImage bimg = new BufferedImage(640,480,BufferedImage.TYPE_INT_RGB);
      //myGraphics = bimg.createGraphics();
      //myWindow = new JFrame("Buffered Image");
      //myWindow.getContentPane().add(new JLabel(new ImageIcon(bimg)));
      //myWindow.setVisible(true);
      height = myPicture.getHeight();
      width = myPicture.getWidth();
    };

    // accessor methods
    public double x()           { return x;           }
    public double y()           { return y;           }
    public double heading() { return heading; }
   

    public void setColor(Color color) {
        foreground = color;
        myGraphics.setColor(foreground);
    }

    public void fly(double x, double y) {
        this.x = x;
        this.y = y;
    }

    public void penUp(){
      penDown = false;
    };
    
    public void penDown(){
      penDown = true;
    };
    
    public boolean pen(){
      return penDown;
    };
    
    public void go(double x, double y) {
        if (penDown)
          myGraphics.draw(new Line2D.Double(this.x, height - this.y, x, height - y));
        //System.out.println("Line:"+this.x+(height - this.y)+x+(height-y));
        this.x = x;
        this.y = y;
    }

    // draw w-by-h rectangle, centered at current location
    public void spot(double w, double h) {
        myGraphics.fill(new Rectangle2D.Double(x - w/2, height - y - h/2, w, h));
    }

    // draw circle of diameter d, centered at current location
    public void spot(double d) {
       if (d <= 1) myGraphics.drawRect((int) x, (int) (height - y), 1, 1);
       else myGraphics.fill(new Ellipse2D.Double(x - d/2, height - y - d/2, d, d));
    }


    // draw spot using jpeg/gif - fix to be centered at (x, y)
    public void spot(String s) {
        Picture spotPicture = new Picture(s);
        Image image = spotPicture.getImage();
 
        int w = image.getWidth(null);
        int h = image.getHeight(null);
   
        // center of rotation is x, height - y
        myGraphics.rotate(Math.toRadians(heading), x, height - y);
        myGraphics.drawImage(image, (int) (x - w/2.0), (int) (height - y - h/2.0), null);
        myGraphics.rotate(Math.toRadians(-heading), x, height - y);
    }

    // draw spot using gif, centered on (x, y), scaled of size w-by-h
    public void spot(String s, double w, double h) {
        Picture spotPicture = new Picture(s);
        Image image = spotPicture.getImage();

        myGraphics.rotate(Math.toRadians(heading), x, height - y);
        myGraphics.drawImage(image, (int) (x - w/2.0), (int) (height - y - h/2.0),
                                   (int) w, (int) h, null);
        myGraphics.rotate(Math.toRadians(-heading), x, height - y);
    }

    public void pixel(int x, int y) {
        myGraphics.drawRect(x, height - y, 1, 1);
    }

    // rotate counterclockwise in degrees
    public void turn(double angle) { heading += angle; }

    // walk forward 
    public void forward(double d) {
        double oldx = x;
        double oldy = y;
        x += d * Math.cos(Math.toRadians(heading));
        y += d * Math.sin(Math.toRadians(heading));
        if (penDown)
          myGraphics.draw(new Line2D.Double(x, height - y, oldx, height - oldy));
        //System.out.println("Line"+x+","+(height-y)+","+oldx+","+(height-oldy));
    }

    // walk forward with pen up
    public void flyForward(double d) {
        x += d * Math.cos(Math.toRadians(heading));
        y += d * Math.sin(Math.toRadians(heading));
    }

    // write the given string in the current font
    public void write(String s) {
        FontMetrics metrics = myGraphics.getFontMetrics();
        int w = metrics.stringWidth(s);
        int h = metrics.getHeight();
        myGraphics.drawString(s, (float) (x - w/2.0), (float) (height - y + h/2.0));
    }

    // write the given string in the given font
    public void write(String s, Font f) {
        myGraphics.setFont(f);
        write(s);
    }


}
